/***********************************************************************/
/*                                                                     */
/*                        Caml examples                                */
/*                                                                     */
/*            Pierre Weis                                              */
/*                                                                     */
/*                        INRIA Rocquencourt                           */
/*                                                                     */
/*  Copyright (c) 1994-2015, INRIA                                     */
/*  All rights reserved.                                               */
/*                                                                     */
/*  Distributed under the BSD license.                                 */
/*                                                                     */
/***********************************************************************/
/* The tokens as given by the lexer. */
%token <int> INT
%token PLUS MINUS TIMES DIV
%token LPAREN RPAREN
%token EOL

/* The associativity (left, right or nonassoc) for the operators,
   and the definition of their relative precedence.
   The precedences should be listed in increasing order. */
%left PLUS MINUS        /* the lowest precedence */
%left TIMES DIV         /* a medium precedence */

/* The definition of precedences to associate to grammar rules
   (and not to the operators of the grammar rules) */
%nonassoc UMINUS        /* the highest precedence */

/* Note: UMINUS is not an operator, it is the name of the precedence that
   should be associated to the grammar rule for the unary minus below. */

/* The entry points of the grammar with their associated declared types. */
%start Main             /* the unique entry point */
%type <int> Main

/* The grammar rules. */
%%

/* Each grammar rule has an associated Caml expression to be returned when
   the rule is applicable to the input text. This expression is the text
   enclosed in '{' and '}'; it is named the ``semantic action'' assiciated to
   the grammar rule in Yacc parlance. */ Main: Expr EOL { $1 } ;

Expr:
    INT                     { $1 }
  | LPAREN Expr RPAREN      { $2 }
  | Expr PLUS Expr          { $1 + $3 }
  | Expr MINUS Expr         { $1 - $3 }
  | Expr TIMES Expr         { $1 * $3 }
  | Expr DIV Expr           { $1 / $3 }
  | MINUS Expr %prec UMINUS { - $2 }
;
